@@ -32,12 +32,12 @@ import sys
 import logging
 import base64
 import copy
+import binascii
 from collections import OrderedDict
 import configparser
 import argparse
 from argparse import ArgumentError
 
-# -----------------------------------------------------------------------------
 # global logger variable
 #log = logging.getLogger('argtoolbox')
 #log.setLevel(logging.INFO)
@@ -61,7 +61,6 @@ streamHandler.setFormatter(DEFAULT_LOGGING_FORMAT)
 #streamHandler.setFormatter(DEBUG_LOGGING_FORMAT)
 
 
-# -----------------------------------------------------------------------------
 class DefaultHook(object):
     """
     This class does nothing. This is the default class for creating your own
@@ -76,7 +75,6 @@ class DefaultHook(object):
         pass
 
 
-# -----------------------------------------------------------------------------
 class Base64ElementHook(DefaultHook):
     """This hook is used as a post reading processing in order to convert
     base64 data stored into the config file into plain text data."""
@@ -89,22 +87,26 @@ class Base64ElementHook(DefaultHook):
             try:
                 data = base64.b64decode(elt.value)
                 elt.value = data
-            except TypeError as ex:
-                # pylint: disable-msg=W0621
-                log = logging.getLogger('argtoolbox')
-                if self.warning:
-                    log.warn("current field '%(name)s' is not \
-stored in the configuration file with \
-base64 encoding",
-                             {"name": getattr(elt, "_name")})
-                else:
-                    log.error("current field '%(name)s' is not stored in the \
-configuration file with base64 encoding", {"name":
-                              getattr(elt, "_name")})
-                    raise ex
+            except binascii.Error as ex:
+                self.handle_exception(ex, elt)
+            except ValueError as ex:
+                self.handle_exception(ex, elt)
+
+    def handle_exception(self, ex, elt):
+        """TODO"""
+        log = logging.getLogger('argtoolbox')
+        log.debug(ex)
+        if self.warning:
+            log.warning(("Current field '%(name)s' is not "
+                         "stored in the configuration file with base64 encoding"),
+                        {"name": getattr(elt, "_name")})
+        else:
+            log.error(("Current field '%(name)s' is not stored in the "
+                       "configuration file with base64 encoding"),
+                      {"name": getattr(elt, "_name")})
+            raise TypeError(str(ex))
 
 
-# -----------------------------------------------------------------------------
 class SectionHook(object):
     """
     The SectionHook class is used during the configuration reloading
@@ -137,7 +139,6 @@ class SectionHook(object):
             setattr(self.section, self.attribute, value)
 
 
-# -----------------------------------------------------------------------------
 class Config(object):
     # pylint: disable-msg=R0902
     """This is the entry point, this class will contains all Section and
@@ -154,7 +155,10 @@ class Config(object):
         self.sections = OrderedDict()
         self._default_section = self.add_section(SimpleSection("DEFAULT"))
         self.parser = None
-        self.file_parser = configparser.SafeConfigParser()
+        if sys.version_info[2] <= 2:
+            self.file_parser = configparser.SafeConfigParser()
+        else:
+            self.file_parser = configparser.ConfigParser()
 
     def add_section(self, section):
         """Add a new Section object to the config. Should be a subclass of
@@ -189,9 +193,14 @@ class Config(object):
             if isinstance(self.config_file, str):
                 discoveredFileList = self.file_parser.read(self.config_file)
             else:
-                discoveredFileList = self.file_parser.readfp(
-                    self.config_file,
-                    "file descriptor")
+                if sys.version_info[2] < 2:
+                    discoveredFileList = self.file_parser.readfp(
+                        self.config_file,
+                        "file descriptor")
+                else:
+                    discoveredFileList = self.file_parser.read_file(
+                        self.config_file,
+                        "file descriptor")
         else:
             defaultFileList = []
             defaultFileList.append(self.prog_name + ".cfg")
@@ -284,11 +293,14 @@ class Config(object):
             log = logging.getLogger('argtoolbox')
             log.debug("reloading configuration ...")
             if args.config_file:
-                self.file_parser = configparser.SafeConfigParser()
+                if sys.version_info[2] <= 2:
+                    self.file_parser = configparser.SafeConfigParser()
+                else:
+                    self.file_parser = configparser.ConfigParser()
                 discoveredFileList = self.file_parser.read(args.config_file)
-                log.debug("discoveredFileList: " + str(discoveredFileList))
+                log.debug("discoveredFileList: %s", discoveredFileList)
             for s in list(self.sections.values()):
-                log.debug("loading section : " + s.get_section_name())
+                log.debug("loading section : %s", s.get_section_name())
                 s.reset()
                 s.load(self.file_parser)
             log.debug("configuration reloaded.")
@@ -335,7 +347,6 @@ class Config(object):
             log.debug("config file generation completed : " + str(output))
 
 
-# -----------------------------------------------------------------------------
 class _AbstractSection(object):
     """This class is the parent class of all Section classes. You can not use
     it, you must implement abstract methods.
@@ -349,6 +360,15 @@ class _AbstractSection(object):
         self._suffix = suffix
         self._required = required
 
+    @property
+    def name(self):
+        """This method will return the name of the current element"""
+        return self._name
+    @name.setter
+    def name(self, name):
+        """TODO"""
+        self._name = name
+
     def get_key_name(self):
         """This method return the name of the section, it Should be unique
         because it is used as a key or identifier."""
@@ -403,7 +423,6 @@ class _AbstractSection(object):
             f.write("\n")
 
 
-# -----------------------------------------------------------------------------
 class _Section(_AbstractSection):
     """Simple abstract section object, container for Elements"""
     def __init__(self, *args, **kwargs):
@@ -415,7 +434,7 @@ class _Section(_AbstractSection):
         will be used as an identifier."""
         if not isinstance(elt, Element):
             raise TypeError("argument should be a subclass of Element")
-        self.elements[elt.get_name()] = elt
+        self.elements[elt.name] = elt
         return elt
 
     def add_element_list(self, elt_list, **kwargs):
@@ -472,7 +491,6 @@ class _Section(_AbstractSection):
         f.write("\n")
 
 
-# -----------------------------------------------------------------------------
 class SimpleSection(_Section):
     """A simple section object. This class is used to declare a section that
     should be into the configuration file.
@@ -495,7 +513,6 @@ class SimpleSection(_Section):
         return res
 
 
-# -----------------------------------------------------------------------------
 class SubSection(_Section):
     """ TODO """
 
@@ -524,7 +541,6 @@ class SubSection(_Section):
         return newone
 
 
-# -----------------------------------------------------------------------------
 class ListSection(_AbstractSection):
     """ TODO """
     def __init__(self, name, *args, **kwargs):
@@ -566,7 +582,6 @@ class ListSection(_AbstractSection):
         return res
 
 
-# -----------------------------------------------------------------------------
 # warning| [R0902, Element] Too many instance attributes (13/7)
 # pylint: disable-msg=R0902
 class Element(object):
@@ -580,7 +595,7 @@ class Element(object):
 
     # pylint: disable-msg=R0913
     def __init__(self, name, e_type=str, required=False, default=None,
-                 conf_hidden=False, conf_required=False, desc=None,
+                 conf_hidden=False, conf_required=False, desc=None, dest=None,
                  hooks=None, hidden=False, e_type_exclude=False):
         """Information about how to declare a element which will be load from a
         configuration file.
@@ -608,6 +623,9 @@ class Element(object):
 
     - desc -- Description used into the configuration file and argparse.
 
+    - desc -- name of the argparse variable name in Namespace.
+        Default: name
+
     - conf_hidden -- The current attribute will not be used during
     configuration file generation.
 
@@ -626,6 +644,7 @@ class Element(object):
         self._required = required
         self.default = default
         self._desc = desc
+        self._dest = dest
         self.conf_hidden = conf_hidden
         self.conf_required = conf_required
         self._desc_for_config = None
@@ -661,6 +680,11 @@ class Element(object):
         """TODO"""
         self._value = value
 
+    @property
+    def name(self):
+        """This method will return the name of the current element"""
+        return self._name
+
     def get_name(self):
         """This method will return the name of the current element"""
         return self._name
@@ -670,23 +694,16 @@ class Element(object):
         representation of the current object. Every lines could be
         prefixed and suffixed.
         """
-        res = []
+        value = self.value
         if self.hidden:
-            res.append(prefix + " - " + str(self._name)
-                       + " : xxxxxxxx" + suffix)
-        else:
-            default = self.default
-            if default is None:
-                default = " - "
-            a = prefix + " - "
-            a += str(self._name) + " : "
-            if isinstance(default, str):
-                a += default
-            else:
-                a += str(default)
-            a += suffix
-            res.append(a)
-        return res
+            value = "xxxxxxxx"
+        line = "{prefix} - {name} : {value}{suffix}".format(
+            prefix=prefix,
+            name=self._name,
+            value=value,
+            suffix=suffix
+        )
+        return line
 
     def __str__(self):
         return "".join(self.get_representation())
@@ -810,6 +827,8 @@ config file." % {"name": self._name}
             else:
                 ret["required"] = True
         ret["dest"] = self._name
+        if self._dest:
+            ret["dest"] = self._dest
         if not self.e_type_exclude:
             if self.e_type == int or self.e_type == float:
                 # Just override argparse.add_argument 'type' parameter for int or float.
@@ -852,7 +871,6 @@ config file." % {"name": self._name}
         f.write("\n")
 
 
-# -----------------------------------------------------------------------------
 class ElementWithSubSections(Element):
     """ This class extends the default class Element. It offers you the power
     to add sections (SubSection) inside a element.
@@ -861,19 +879,41 @@ class ElementWithSubSections(Element):
     Section :
         Element1 : name = port, value = 389
         Element2 : name = address, value = 127.0.0.1
-        ElementWithSubSections
+        Element3 (ElementWithSubSections): name = nested_section_name, value = nested1
             SubSection
-                Element1
-                Element2
+                Element4 : name = proto, value = tcp
+                Element5 : name = version, value = 3
+    ---
+    [DEFAULT]
+    port=369
+    address=127.0.0.1
+    nested_section_name=nested1
+
+    [nested1]
+    proto=tcp
+    version=3
+    ---
+    default = self.config.get_default_section()
+    section = SubSection()
+    section.add_element(Element('trigger', conf_required=True))
+    section.add_element(Element('hook', conf_required=True))
+    default.add_element(ElementWithSubSections('group_cfg', section))
     """
 
-    def __init__(self, *args, **kwargs):
-        super(ElementWithSubSections, self).__init__(*args, **kwargs)
+    def __init__(self, name, subsection, *args, **kwargs):
+        super(ElementWithSubSections, self).__init__(name, *args, **kwargs)
         self.e_type = str
+        # FIXME: why using a list if we have only one section ?
         self.sections = OrderedDict()
+        self.add_section(subsection)
+
+    @property
+    def section(self):
+        """This method will return the name of the current element"""
+        return self.sections[self._name]
 
     def get_representation(self, prefix="", suffix="\n"):
-        res = ['\n']
+        res = []
         temp_line = prefix + " - " + str(self._name) + " : "
         if self.hidden:
             temp_line += "xxxxxxxx" + suffix
@@ -887,14 +927,16 @@ class ElementWithSubSections(Element):
         return res
 
     def add_section(self, section):
-        """You can add section inside a Element, the section must be a
+        """You can add a section inside a Element, the section must be a
         subclass of SubSection. You can use this class to represent a tree.
+        Be careful, you can't add more than one section.
         """
 
         if not issubclass(section.__class__, SubSection):
             raise TypeError("Argument should be a subclass of SubSection, \
                              not :" + str(section.__class__))
-        self.sections[section.name] = section
+        # we always use the element name as the unique section name.
+        self.sections[self.name] = section
         return section
 
     def load(self, file_parser, section_name):
@@ -906,8 +948,7 @@ class ElementWithSubSections(Element):
         self.post_load()
 
 
-# -----------------------------------------------------------------------------
-class ElementWithRelativeSubSection(ElementWithSubSections):
+class ElementWithRelativeSubSection(Element):
 
     """ This class extends the default class Element. It offers you the power
     to add sections (SubSection) inside a element.
@@ -942,21 +983,47 @@ class ElementWithRelativeSubSection(ElementWithSubSections):
             SubSection : sec_test1
                 - Element4
                 - Element5
-                - Element5
+                - Element6
             SubSection : sec_test2
                 - Element4
                 - Element5
-                - Element5
+                - Element6
             SubSection : sec_test3
                 - Element4
                 - Element5
-                - Element5
+                - Element6
+    ---
+    [DEFAULT]
+    port=369
+    address=127.0.0.1
+    list_of_section_name=sec_test1 sec_test2 sec_test3
 
+    [sec_test1]
+    elt4_field=tcp
+    elt5_field=3
+    elt6_field=aaaa
+
+    [sec_test2]
+    elt4_field=tcp
+    elt5_field=3
+    elt6_field=aaaa
+
+    [sec_test3]
+    elt4_field=tcp
+    elt5_field=3
+    elt6_field=aaaa
+    ---
+    section = SubSection()
+    section.add_element(Element('elt4_field', conf_required=True))
+    section.add_element(Element('elt5_field', conf_required=True))
+    section.add_element(Element('elt6_field', conf_required=True))
+    default.add_element(ElementWithRelativeSubSection('list_of_section_name', section))
     """
 
-    def __init__(self, name, rss, **kwargs):
-        super(ElementWithRelativeSubSection, self).__init__(name, **kwargs)
+    def __init__(self, name, rss, *args, **kwargs):
+        super(ElementWithRelativeSubSection, self).__init__(name, *args, **kwargs)
         self.e_type = list
+        self.sections = OrderedDict()
         if not issubclass(rss.__class__, SubSection):
             raise TypeError("Argument should be a subclass of SubSection, \
                             not :" + str(_Section.__class__))
@@ -983,26 +1050,19 @@ class ElementWithRelativeSubSection(ElementWithSubSections):
         self.post_load()
 
     def get_representation(self, prefix="", suffix="\n"):
-        res = ['\n']
-        res.append('#')
+        res = []
         temp_line = prefix + " - " + str(self._name) + " : "
         if self.hidden:
             temp_line += "xxxxxxxx" + suffix
         else:
             temp_line += str(self.value) + suffix
         res.append(temp_line)
-
-        res.append("".join(self.rss.get_representation(prefix + "#\t")))
-        res.append('\n')
         if len(self.sections) > 0:
             for elt in list(self.sections.values()):
-                res.append('\n')
                 res.append("".join(elt.get_representation(prefix + "\t")))
-            res.append('\n')
         return res
 
 
-# -----------------------------------------------------------------------------
 class DefaultCommand(object):
     """This class do nothing, this is just the default structure to implement
     your own class. Use this class as the base for every the command line
@@ -1038,7 +1098,6 @@ class DefaultCommand(object):
         return []
 
 
-# -----------------------------------------------------------------------------
 class TestCommand(DefaultCommand):
     """Just a simple command, using the default command class."""
 
@@ -1062,7 +1121,6 @@ class TestCommand(DefaultCommand):
         print("")
 
 
-# -----------------------------------------------------------------------------
 class DefaultCompleter(object):
     """ TODO
     """
@@ -1071,13 +1129,13 @@ class DefaultCompleter(object):
 
     def __call__(self, prefix, **kwargs):
         from argcomplete import debug
+        from argcomplete import warn
         try:
             debug("\n------------ DefaultCompleter -----------------")
             debug("Kwargs content :")
             for i, j in list(kwargs.items()):
                 debug("key : " + str(i))
                 debug("\t - " + str(j))
-            debug("\n------------ DefaultCompleter -----------------\n")
 
             args = kwargs.get('parsed_args')
             # pylint: disable-msg=W0612
@@ -1087,9 +1145,15 @@ class DefaultCompleter(object):
             # getting form args the current Command and looking for a method
             # called by default 'complete'. See __init__ method. The method
             # name is store in the class member called self.func_name
-            fn = getattr(args.__func__, self.__name__, None)
+            debug("-------")
+            debug("__func__:" + str(args.__func__))
+            debug("func_name:" + str(self.func_name))
+            debug("\n------------ DefaultCompleter -----------------\n")
+            fn = getattr(args.__func__, self.func_name, None)
             if fn:
                 return fn(args, prefix)
+            warn("ERROR: Can not find completion function inside the __func__ object !")
+            return []
 
         # pylint: disable-msg=W0703
         except Exception as e:
@@ -1101,7 +1165,6 @@ class DefaultCompleter(object):
             return ["comlete-error"]
 
 
-# -----------------------------------------------------------------------------
 class DefaultProgram(object):
     """ TODO """
 
@@ -1212,7 +1275,6 @@ class DefaultProgram(object):
         return args
 
 
-# -----------------------------------------------------------------------------
 class BasicProgram(object):
     """ TODO """
 
@@ -1335,7 +1397,6 @@ class BasicProgram(object):
             sys.exit(1)
 
 
-# -----------------------------------------------------------------------------
 def query_yes_no(question, default="yes"):
     """Just prompt the user for a yes/no question"""
     res = _query_yes_no(question, default)
@@ -1345,7 +1406,6 @@ def query_yes_no(question, default="yes"):
         return False
 
 
-# -----------------------------------------------------------------------------
 def _query_yes_no(question, default="yes"):
     """Ask a yes/no question via raw_input() and return their answer.
 
